#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
The first step of this script is to run on bash will move to python eventually
Usage:
  ccspy_anat_03_postfs [options]

Options:
  --CCS_DIR PATH     The directory for CCS
  --SUBJECTS_DIR PATH     The directory for SUBJECTS
  --subject PATH       The subject's id
"""

from docopt import docopt
import os
import logging
import ccspy.utils.run_command_util as run_command
import ccspy.utils.logging_config_util as logging_config_util
import ccspy.utils.file_util as file_util


def postfs(CCS_DIR, SUBJECTS_DIR, subject, logger):

    anat_dir = os.path.join(CCS_DIR, subject, 'anat')
    reg_dir = os.path.join(anat_dir, 'reg')
    seg_dir = os.path.join(anat_dir, 'segment')

    file_util.make_dir(reg_dir)
    file_util.make_dir(seg_dir)

    # generate copy brainmask

    # copy orig.mgz
    if not os.path.exists(os.path.join(anat_dir, 'T1_crop_sanlm_fs_rpi.nii.gz')):
        command = ['mri_convert', '-it', 'mgz', os.path.join(SUBJECTS_DIR, subject, 'mri/orig.mgz'),
                   '-ot', 'nii', os.path.join(anat_dir, 'T1_crop_sanlm_fs.nii.gz')]
        run_command.run(command)

    if not os.path.exists(os.path.join(seg_dir, 'brainmask.nii.gz')):
        command = ['mri_convert', '-it', 'mgz', os.path.join(SUBJECTS_DIR, subject, 'mri/brainmask.mgz'),
                   '-ot', 'nii', os.path.join(seg_dir, 'brainmask.nii.gz')]
        run_command.run(command)

    # prepare anatomical images
    file_util.remove_file(os.path.join(reg_dir, 'highres_head.nii.gz'))

    command = ['mv', os.path.join(anat_dir, 'T1_crop_sanlm_fs.nii.gz'),
               os.path.join(reg_dir, 'highres_head.nii.gz')]
    run_command.run(command)

    # clean voxels manually edited in freesurfer (assigned value 1)
    command = ['fslmaths', os.path.join(seg_dir, 'brainmask.nii.gz'),
               '-thr', '2', os.path.join(seg_dir, 'brainmask.nii.gz')]
    run_command.run(command)

    command = ['fslmaths', os.path.join(reg_dir, 'highres_head.nii.gz'),
               '-mas', os.path.join(seg_dir, 'brainmask.nii.gz'),
               os.path.join(reg_dir, 'highres.nii.gz')]
    run_command.run(command)

    # command = ['cd', reg_dir]
    # run_command.run(command)
    os.chdir(reg_dir)

    # 1. copy standard (we provide two reg pipelines: FSL and Freesurfer,
    # the latter was done in recon-all automatically)
    out = run_command.run("echo ${FSLDIR}")
    out = out.replace('\n', '').replace('\r', '')

    standard_head = os.path.join(out, 'data/standard/MNI152_T1_2mm.nii.gz')
    standard = os.path.join(out, 'data/standard/MNI152_T1_2mm_brain.nii.gz')
    standard_mask = os.path.join(out, 'data/standard/MNI152_T1_2mm_brain_mask_dil.nii.gz')

    # command = ['standard_head=${FSLDIR}/data/standard/MNI152_T1_2mm.nii.gz']
    # run_command.run(command)
    #
    # run_command.run("echo ${standard_head}")
    #
    # command = ['standard=${FSLDIR}/data/standard/MNI152_T1_2mm_brain.nii.gz']
    # run_command.run(command)
    #
    # command = ['standard_mask=${FSLDIR}/data/standard/MNI152_T1_2mm_brain_mask_dil.nii.gz']
    # run_command.run(command)

    # 2. FLIRT T1->STANDARD
    logger.info("########## performing FLIRT T1 -> STANDARD ##########")
    command = ['fslreorient2std', 'highres.nii.gz', 'highres_rpi.nii.gz']
    run_command.run(command)

    # not used, just test for future use
    command = ['fslreorient2std', 'highres_head.nii.gz', 'highres_head_rpi.nii.gz']
    run_command.run(command)

    command = ['flirt', '-ref', standard, '-in', 'highres_rpi', '-out', 'highres_rpi2standard',
               '-omat', 'highres_rpi2standard.mat', '-cost', 'corratio', '-searchcost',
               'corratio', '-dof', '12', '-interp', 'trilinear']
    run_command.run(command)

    # 3. create mat file for conversion from standard to high res
    command = ['fslreorient2std', 'highres.nii.gz', '>', 'reorient2rpi.mat']
    run_command.run(command)

    command = ['convert_xfm', '-omat', 'highres2standard.mat', '-concat',
               'highres_rpi2standard.mat', 'reorient2rpi.mat']
    run_command.run(command)

    command = ['convert_xfm', '-inverse', '-omat', 'standard2highres.mat', 'highres2standard.mat']
    run_command.run(command)

    # 3. FNIRT
    logger.info("########## performing nolinear registration ... ##########")
    command = ['fnirt', '--in=highres_head', '--aff=highres2standard.mat',
               '--cout=highres2standard_warp', '--iout=fnirt_highres2standard',
               '--jout=highres2standard_jac', '--config=T1_2_MNI152_2mm',
               '--ref='+standard_head, '--refmask='+standard_mask,
               '--warpres=10,10,10', '>', 'warnings.fnirt']
    run_command.run(command)

    path = os.path.join(reg_dir, 'warnings.fnirt')
    if os.path.getsize(path) > 0:

        command = ['mv', 'fnirt_highres2standard.nii.gz', 'fnirt_highres2standard_wres10.nii.gz']
        run_command.run(command)

        command = ['fnirt', '--in=highres_head', '--aff=highres2standard.mat',
                   '--cout=highres2standard_warp', '--iout=fnirt_highres2standard',
                   '--jout=highres2standard_jac', '--config=T1_2_MNI152_2mm',
                   '--ref='+standard_head, '--refmask='+standard_mask,
                   '--warpres=20,20,20']
        run_command.run(command)

    else:
        file_util.remove_file(os.path.join(reg_dir, "warnings.fnirt"))

    # command = ['cd', '${cwd}']
    # run_command.run(command)
    os.chdir('/')


def main():

    arguments = docopt(__doc__)
    CCS_DIR = arguments['--CCS_DIR']
    SUBJECTS_DIR = arguments['--SUBJECTS_DIR']
    subject = arguments['--subject']

    logging_config_util.config()
    logger = logging.getLogger(__name__)

    logger.info(CCS_DIR)
    logger.info(SUBJECTS_DIR)
    logger.info(subject)

    if CCS_DIR is None or SUBJECTS_DIR is None or subject is None:
        logger.error("\nUsage: {} {} CCS_DIR SUBJECTS_DIR subject".format(__file__, __name__))
        return

    postfs(CCS_DIR, SUBJECTS_DIR, subject, logger)




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
The first step of this script is to run on bash will move to python eventually
Usage:
  ccspy_anat [options]

Options:
  --CCS_DIR PATH     The directory for CCS
  --SUBJECTS_DIR PATH     The directory for SUBJECTS
  --subject PATH       The subject's id
"""

import os
import logging
from docopt import docopt
import ccspy.utils.run_command_util as run_command_util
import ccspy.utils.logging_config_util as lcu
import ccspy.ccspy_anat_01_pre_freesurfer as ccs_anat_01
import ccspy.ccspy_anat_02_freesurfer as ccs_anat_02
import ccspy.ccspy_anat_03_postfs as ccs_anat_03


def main():

    arguments = docopt(__doc__)
    CCS_DIR = arguments['--CCS_DIR']
    SUBJECTS_DIR = arguments['--SUBJECTS_DIR']
    subject = arguments['--subject']

    lcu.config()
    logger = logging.getLogger(__name__)

    logger.info("CCS_DIR: " + CCS_DIR)
    logger.info("SUBJECTS_DIR: " + SUBJECTS_DIR)
    logger.info("subject: " + subject)

    if CCS_DIR is None or SUBJECTS_DIR is None or subject is None:
        logger.error("\nUsage: {} {} CCS_DIR SUBJECTS_DIR subject".format(__file__, __name__))
        return

    ccs_anat_01.pre_freesurfer(CCS_DIR, subject, logger)
    ccs_anat_02.freesurfer(CCS_DIR, SUBJECTS_DIR, subject, logger)
    ccs_anat_03.postfs(CCS_DIR, SUBJECTS_DIR, subject, logger)


if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   say_hello.py   
@Author  :   dzjin
@Contact :   dzjin5678@163.com
@License :   Nothing


@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/6/1 16:23   dzjin      1.0         None
"""

import os
import logging
import ccspy.utils.logging_config_util as lcu


def main():
    print("Hello ,This is my Python library.")
    # run_command.run('pwd')
    # run_command.run('cksdnvknfdvnfd')
    # print('1')
    # fun_test_return()
    # print('2')
    # fun_test_sys_exit()
    # print('3')

    test_logging_config()


# def fun_test_return():
#     print('fun_test_return')
#     return
#
#
# def fun_test_sys_exit():
#     print('fun_test_sys_exit')
#     sys.exit()


def test_logging_config():

    # logging.basicConfig(filename=os.path.join("d:\python", "out.log"), level=logging.DEBUG)

    lcu.config()
    logger = logging.getLogger(__name__)

    logger.debug("debug")
    logger.info("info")
    logger.warning("warning")
    logger.error("error")
    logger.error(__name__)
    logger.error(os.path.dirname(__file__))


if __name__ == '__main__':
    test_logging_config()




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py.py   
@Author  :   dzjin
@Contact :   dzjin5678@163.com
@License :   Nothing


@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/6/15 10:04   dzjin      1.0         None
"""

# import lib


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   run_command_util.py
@Author  :   dzjin
@Contact :   dzjin5678@163.com
@License :   Nothing


@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/6/15 10:05   dzjin      1.0         None
"""

import os
import sys
import subprocess
import logging


def run(command, env=None):
    """
    运行命令行命令
    :param command: 命令
    :param env: 字符串环境
    :return: 返回码
    """

    logger = logging.getLogger(__name__)

    merged_env = os.environ
    if env:
        merged_env.update(env)

    if type(command) is list:
        command = ' '.join(command)
    logger.info("[Running COMMAND]\n "+command+"\n")

    p = subprocess.Popen(command, shell=True,
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=merged_env)
    out, err = p.communicate()
    out = out.decode('utf-8')
    err = err.decode('utf-8')

    if len(out) > 0:
        logger.info(out)
    if len(err) > 0:
        logger.error(err)

    if p.returncode:
        # 指令运行出错，终止程序的执行
        logger.error('COMMAND: {} , \nFailed with returncode {}'.format(command, p.returncode))
        sys.exit()
    else:
        # 程序运行正常，正常返回
        return out



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
The first step of this script is to run on bash will move to python eventually
Usage:
  ccspy_anat_01_pre_freesurfer [options]

Options:
  --CCS_DIR PATH     The directory for CCS
  --SUBJECTS_DIR PATH     The directory for SUBJECTS
  --subject PATH       The subject's id
"""

import os
import logging
import ants
from docopt import docopt
import ccspy.utils.run_command_util as run_command_util
import ccspy.utils.logging_config_util as logging_config_util


def pre_freesurfer(CCS_DIR, subject, logger):

    anat_dir = os.path.join(CCS_DIR, subject, 'anat')

    path = os.path.join(CCS_DIR, subject, "anat/T1.nii.gz")
    if not os.path.exists(path):
        logger.error("\nCouldn't find T1 image {} please check your data".format(path))
        return

    # 1.crop the image
    if not os.path.exists(os.path.join(anat_dir, "T1_crop.nii.gz")):
        logger.info("Croping image")
        command = ["robustfov", "-i",
                   os.path.join(anat_dir, "T1.nii.gz"), "-r", os.path.join(anat_dir, "T1_crop.nii.gz")]
        run_command_util.run(command, env=None)

    # 2.using SANLM to denoise image
    if not os.path.exists(os.path.join(anat_dir, "T1_crop_sanlm.nii.gz")):

        logger.info("desonising image")

        # use python ants package antspyx replace matlab denoise
        input_image = ants.image_read(os.path.join(anat_dir, 'T1_crop.nii.gz'))
        output_image = ants.denoise_image(input_image, v=1)
        ants.image_write(output_image, os.path.join(anat_dir, 'T1_crop_sanlm.nii.gz'))

        # path = os.path.join(anat_dir, "denoise")
        # if not os.path.exists(path):
        #     os.makedirs(path)
        #
        # command = ['mri_convert', '-i', os.path.join(anat_dir, 'T1_crop.nii.gz'),
        #            '-o', os.path.join(anat_dir, 'denoise', 'T1_crop.nii')]
        # run_command_util.run(command)

        # command = ['matlab', '-nodesktop', '-nosplash', '-nojvm', '-r',
        #            "\"data='{}/denoise/T1_crop.nii';cat_vol_sanlm(struct('data',data));quit\"".format(anat_dir)]
        # run_command_util.run(command)
        #
        # command = ['mri_convert', '-i', os.path.join(anat_dir, 'denoise', 'sanlm_T1_crop.nii'),
        #            '-o', os.path.join(anat_dir, 'T1_crop_sanlm.nii.gz')]
        # run_command_util.run(command)

        # remove denoise dirs
        # os.removedirs(os.path.join(anat_dir, 'denoise'))

    # 3.deepbet to making brain mask
    if not os.path.exists(os.path.join(anat_dir, 'T1_crop_sanlm_pre_mask.nii.gz')):
        logger.info("making mask by deep_bet")

        command = ['docker', 'run', '-v', '{}:/data'.format(anat_dir), '-v', '${CCS_APP}/Models:/Models',
                   '-v', '{}:/output'.format(anat_dir), '--privileged=true',
                   'sandywangrest/deepbet', 'muSkullStrip.py',
                   '-in', '/data/T1_crop_sanlm.nii.gz', '-model', '/Models/model-04-epoch',
                   '-out', '/output']
        run_command_util.run(command)

        path = os.path.join(anat_dir, "qc")
        if not os.path.exists(path):
            os.makedirs(path)

        command = ['overlay', '1', '1', '{}/T1_crop_sanlm.nii.gz'.format(anat_dir),
                   '-a', '{}/T1_crop_sanlm_pre_mask.nii.gz'.format(anat_dir),
                   '1', '1', '{}/qc/T1_rendermask.nii.gz'.format(anat_dir)]
        run_command_util.run(command)

        command = ['slicer', '{}/qc/T1_rendermask.nii.gz'.format(anat_dir),
                   '-S', '10', '1200', '{}/qc/mask.png'.format(anat_dir)]
        run_command_util.run(command)

        title = subject + '.anat.UNet.skullstrip'
        command = ['convert', '-font', 'helvetica', '-fill', 'white', '-pointsize', '36',
                   '-draw', "\"text 30,50 '{}'\"".format(title),
                   "{}/qc/mask.png".format(anat_dir), "{}/qc/mask.png".format(anat_dir)]
        run_command_util.run(command)

        os.remove(os.path.join(anat_dir, "qc", "T1_rendermask.nii.gz"))


def main():

    arguments = docopt(__doc__)
    CCS_DIR = arguments['--CCS_DIR']
    # SUBJECTS_DIR = arguments['--SUBJECTS_DIR']
    subject = arguments['--subject']

    logging_config_util.config()
    logger = logging.getLogger(__name__)

    logger.info("CCS_DIR: " + CCS_DIR)
    # logger.info("SUBJECTS_DIR: " + SUBJECTS_DIR)
    logger.info("subject: " + subject)

    if CCS_DIR is None or subject is None:
        logger.error("\nUsage: {} {} CCS_DIR SUBJECTS_DIR subject".format(__file__, __name__))
        return

    pre_freesurfer(CCS_DIR, subject, logger)











#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   setup.py   
@Author  :   dzjin
@Contact :   dzjin5678@163.com
@License :   Nothing


@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/6/1 16:25   dzjin      1.0         None
"""

import setuptools

setuptools.setup(
    name="ccspy",  # Replace with your own username
    version="0.0.16",
    author="dzjin",
    author_email="dzjin5678@163.com",
    description="A CCS PACKAGE",
    long_description="A CCS PACKAGE",
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    data_files=[('', ['README.md'])],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        'Operating System :: POSIX :: Linux',
    ],
    entry_points={
        'console_scripts': [
            'ccspy_anat=ccspy.ccspy_anat:main',
            'ccspy_anat_01_pre_freesurfer=ccspy.ccspy_anat_01_pre_freesurfer:main',
            'ccspy_anat_02_freesurfer=ccspy.ccspy_anat_02_freesurfer:main',
            'ccspy_anat_03_postfs=ccspy.ccspy_anat_03_postfs:main',
            'ccspy_say_hello=ccspy.say_hello:main'
        ],
    },
    install_requires=[
            'docopt',
            'antspyx',
    ],
    include_package_data=True,
    python_requires='>=3.0',
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   logging_config_util.py   
@Author  :   dzjin
@Contact :   dzjin5678@163.com
@License :   Nothing


@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/6/18 16:28   dzjin      1.0         None
"""

import os
import json
import logging.config


def config():
    path = os.path.join(os.path.dirname(__file__), "logging_config.json")
    if os.path.exists(path):
        with open(path, 'r') as f:
            logging.config.dictConfig(json.load(f))
    else:
        logging.basicConfig(level=logging.INFO)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   file_util.py
@Author  :   dzjin
@Contact :   dzjin5678@163.com
@License :   Nothing


@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/6/18 11:05   dzjin      1.0         None, add make_dir
2021-6-19 17:49   dzjin      1.0         None, add remove file(file and dir)
"""

import os
import sys
import logging


def make_dir(path, suppress_exists_error=False):
    """
    创建文件夹
    :param path: 待创建文件夹路径
    :param suppress_exists_error: 是否忽略FileExistsError
    :return:
    """
    logger = logging.getLogger(__name__)

    try:
        os.makedirs(path)
    except PermissionError:
        # 没有权限创建时，程序强制结束
        logger.error("You do not have permission to write to {}".format(path))
        sys.exit()
    except FileExistsError:
        if suppress_exists_error:
            logger.warning("{} already exists".format(path))
    except OSError:
        logger.error('Could not create directory {}'.format(path))
        sys.exit()


def remove_file(path):
    """
    删除文件或者文件夹
    :param path:
    :return:
    """
    logger = logging.getLogger(__name__)
    if os.path.exists(path):
        try:
            os.remove(path)
        except PermissionError:
            # 没有权限删除时，程序不强制结束
            logger.error("You do not have permission to remove {}".format(path))
        except OSError:
            logger.error('Could not remove file {}'.format(path))
            sys.exit()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
The first step of this script is to run on bash will move to python eventually
Usage:
  ccspy_anat_01_pre_freesurfer [options]

Options:
  --CCS_DIR PATH     The directory for CCS
  --SUBJECTS_DIR PATH     The directory for SUBJECTS
  --subject PATH       The subject's id
"""

from docopt import docopt
import os
import logging
import ccspy.utils.run_command_util as run_command_util
import ccspy.utils.logging_config_util as logging_config_util
import ccspy.utils.file_util as file_util


def freesurfer(CCS_DIR, SUBJECTS_DIR, subject, logger):
    anat_dir = os.path.join(CCS_DIR, subject, 'anat')

    path = os.path.join(anat_dir, "T1_crop_sanlm.nii.gz")
    if not os.path.exists(path):
        logger.error("\nCouldn't find T1_crop_sanlm.nii.gz image {} please check your data".format(path))
        return

    file_util.make_dir(os.path.join(SUBJECTS_DIR, subject, 'mri/orig'))

    # convert T1_crop_sanlm to 001.mgz
    command = ['mri_convert', '-i', os.path.join(anat_dir, 'T1_crop_sanlm.nii.gz'),
               '-o', os.path.join(SUBJECTS_DIR, subject, 'mri/orig/001.mgz')]
    run_command_util.run(command)

    # first stage of recall-all
    command = ['recon-all', '-s', subject, '-autorecon1', '-gcut', '-parallel']
    run_command_util.run(command)

    # convert brainmask
    logger.info("\n########## Changing orig brainmask by the deep_bet brainmask ##########")
    command = ['mri_convert', os.path.join(SUBJECTS_DIR, subject, 'mri/T1.mgz'),
               os.path.join(SUBJECTS_DIR, subject, 'mri/T1.nii.gz')]
    run_command_util.run(command)

    command = ['3dresample', '-master', os.path.join(SUBJECTS_DIR, subject, 'mri/T1.nii.gz'),
               '-inset', os.path.join(anat_dir, 'T1_crop_sanlm_pre_mask.nii.gz'),
               '-prefix', os.path.join(SUBJECTS_DIR, subject, 'mri/mask.nii.gz')]
    run_command_util.run(command)

    command = ['fslmaths', os.path.join(SUBJECTS_DIR, subject, 'mri/T1.nii.gz'),
               '-mas', os.path.join(SUBJECTS_DIR, subject, 'mri/mask.nii.gz'),
               os.path.join(SUBJECTS_DIR, subject, 'mri/brainmask.nii.gz')]
    run_command_util.run(command)

    command = ['mv', os.path.join(SUBJECTS_DIR, subject, 'mri/brainmask.mgz'),
               os.path.join(SUBJECTS_DIR, subject, 'mri/brainmask.fsinit.mgz')]
    run_command_util.run(command)

    command = ['mri_convert', '-i', os.path.join(SUBJECTS_DIR, subject, 'mri/brainmask.nii.gz'),
               '-o', os.path.join(SUBJECTS_DIR, subject, 'mri/brainmask.mgz')]
    run_command_util.run(command)

    # recon-all 2 3
    command = ['recon-all', '-s', subject, '-autorecon2', '-autorecon3', '-parallel']
    run_command_util.run(command)


def main():

    arguments = docopt(__doc__)
    CCS_DIR = arguments['--CCS_DIR']
    SUBJECTS_DIR = arguments['--SUBJECTS_DIR']
    subject = arguments['--subject']

    logging_config_util.config()
    logger = logging.getLogger(__name__)

    logger.info(CCS_DIR)
    logger.info(SUBJECTS_DIR)
    logger.info(subject)

    if CCS_DIR is None or SUBJECTS_DIR is None or subject is None:
        logger.error("\nUsage: {} {} CCS_DIR SUBJECTS_DIR subject".format(__file__, __name__))
        return

    freesurfer(CCS_DIR, SUBJECTS_DIR, subject, logger)
